@@ -1,8 +1,10 @@
 import logging
+import os
+import platform
 import sys
 from types import MappingProxyType
+from typing import cast
 from os import environ
-from outdated import check_outdated
 from pathlib import Path
 from PySide6.QtWidgets import (
     QApplication,
@@ -18,9 +20,11 @@ from PySide6.QtWidgets import (
 )
 from PySide6.QtCore import (
     QObject,
+    QSettings,
     Qt,
     Signal,
     Slot,
+    QEvent,
 )
 from PySide6.QtGui import (
     QAction,
@@ -28,8 +32,8 @@ from PySide6.QtGui import (
     QGuiApplication,
 )
 
+import datalad
 from datalad import cfg as dlcfg
-from datalad import __version__ as dlversion
 import datalad.ui as dlui
 from datalad.interface.base import Interface
 from datalad.local.wtf import (
@@ -40,6 +44,7 @@ from datalad.utils import chpwd
 
 from .utils import (
     load_ui,
+    open_terminal_at_path,
     render_cmd_call,
 )
 from .datalad_ui import GooeyUI
@@ -48,18 +53,34 @@ from .dataladcmd_ui import GooeyDataladCmdUI
 from .cmd_actions import add_cmd_actions_to_menu
 from .fsbrowser import GooeyFilesystemBrowser
 from .resource_provider import gooey_resources
+from . import utility_actions as ua
+from . import credentials as cred
+from .history_widget import HistoryWidget
+from .metadata_widget import MetadataWidget
+from .property_widget import PropertyWidget
 
 lgr = logging.getLogger('datalad.ext.gooey.app')
 
 
 class GooeyApp(QObject):
+
+    execute_dataladcmd = Signal(str, MappingProxyType, MappingProxyType)
+    configure_dataladcmd = Signal(str, MappingProxyType)
+
     # Mapping of key widget names used in the main window to their widget
     # classes.  This mapping is used (and needs to be kept up-to-date) to look
     # up widget (e.g. to connect their signals/slots)
-    _main_window_widgets = {
+    _widgets = {
         'contextTabs': QTabWidget,
+        'consoleTabs': QTabWidget,
         'cmdTab': QWidget,
-        'propertyBrowser': QTextBrowser,
+        'commandLogTab': QWidget,
+        'metadataTab': QWidget,
+        'metadataTabWidget': MetadataWidget,
+        'historyWidget': HistoryWidget,
+        'helpTab': QWidget,
+        'helpBrowser': QTextBrowser,
+        'propertyWidget': PropertyWidget,
         'fsBrowser': QTreeWidget,
         'commandLog': QPlainTextEdit,
         'errorLog': QPlainTextEdit,
@@ -76,12 +97,12 @@ class GooeyApp(QObject):
         'actionDiagnostic_infos': QAction,
     }
 
-    execute_dataladcmd = Signal(str, MappingProxyType, MappingProxyType)
-    configure_dataladcmd = Signal(str, MappingProxyType)
-
     def __init__(self, path: Path = None):
         super().__init__()
         # bend datalad to our needs
+        # e.g. prevent doc assembly for Python API -- we are better off with
+        # the raw material
+        datalad.enable_librarymode()
         # we cannot handle ANSI coloring
         dlcfg.set('datalad.ui.color', 'off', scope='override', force=True)
 
@@ -94,6 +115,12 @@ class GooeyApp(QObject):
                 'SSH_ASKPASS',
             )
         }
+        if platform.system() == 'Windows':
+            # https://github.com/datalad/datalad-gooey/issues/371
+            # alternative to https://github.com/datalad/datalad-gooey/pull/380
+            self._restore_env['DISPLAY'] = environ.get('DISPLAY')
+            environ['DISPLAY'] = "0:0"
+
         # prevent any terminal-based interaction of Git
         # do it here, not just for command execution to also catch any possible
         # ad-hoc Git calls
@@ -107,7 +134,8 @@ class GooeyApp(QObject):
         self._setup_looknfeel()
 
         self._dlapi = None
-        self._main_window = None
+        self.__main_window = None
+        self.__app_close_requested = False
         self._cmdexec = GooeyDataladCmdExec()
         self._cmdui = GooeyDataladCmdUI(self, self.get_widget('cmdTab'))
 
@@ -142,22 +170,6 @@ class GooeyApp(QObject):
         self._cmdexec.execution_started.connect(self._setup_ongoing_cmdexec)
         self._cmdexec.execution_finished.connect(self._setup_stopped_cmdexec)
         self._cmdexec.execution_failed.connect(self._setup_stopped_cmdexec)
-        # arrange for the dataset menu to populate itself lazily once
-        # necessary
-        self.get_widget('menuDatalad').aboutToShow.connect(
-            self._populate_datalad_menu)
-        self.main_window.actionSetBaseDirectory.triggered.connect(
-            self._set_root_path)
-        self.main_window.actionCheck_for_new_version.triggered.connect(
-            self._check_new_version)
-        self.main_window.actionReport_a_problem.triggered.connect(
-            self._get_issue_template)
-        self.main_window.actionGetHelp.triggered.connect(
-            self._get_help)
-        self.main_window.actionAbout.triggered.connect(
-            self._get_info)
-        self.main_window.actionDiagnostic_infos.triggered.connect(
-            self._get_diagnostic_info)
         # connect the diagnostic WTF helper
         self._cmdexec.results_received.connect(
             self._app_cmdexec_results_handler)
@@ -169,7 +181,35 @@ class GooeyApp(QObject):
         # https://github.com/datalad/datalad-gooey/issues/105
         #self._fsbrowser._tree.currentItemChanged.connect(
         #    lambda cur, prev: self._cmdui.reset_form())
+        self._setup_menus()
 
+        # check if we have an identity. Most of datalad will blow up if not
+        if dlcfg.get('user.name') is None or dlcfg.get('user.email') is None:
+            ua.set_git_identity(self.main_window)
+
+        self._restore_configuration()
+
+    def _setup_menus(self):
+        # arrange for the dataset menu to populate itself lazily once
+        # necessary
+        self.get_widget('menuDatalad').aboutToShow.connect(
+            self._populate_datalad_menu)
+        self.main_window.actionSetBaseDirectory.triggered.connect(
+            self._set_root_path)
+        self.main_window.actionCheck_for_new_version.triggered.connect(
+            lambda: ua.check_new_datalad_version(self))
+        self.main_window.actionReport_a_problem.triggered.connect(
+            lambda: ua.get_issue_template(self.main_window))
+        self.main_window.actionGetHelp.triggered.connect(
+            lambda: ua.get_help(self.main_window))
+        self.main_window.actionAbout.triggered.connect(
+            lambda: ua.show_about_info(self.main_window))
+        self.main_window.actionDiagnostic_infos.triggered.connect(
+            lambda: ua.get_diagnostic_info(self))
+        self.main_window.actionSetAuthorIdentity.triggered.connect(
+            lambda: ua.set_git_identity(self.main_window))
+        self.main_window.actionManageCredentials.triggered.connect(
+            lambda: cred.show_credential_manager(self.main_window))
         # TODO could be done lazily to save in entrypoint iteration
         self._setup_suites()
         self._connect_menu_view(self.get_widget('menuView'))
@@ -183,8 +223,12 @@ class GooeyApp(QObject):
             # but not for internal calls
             # https://github.com/datalad/datalad-gooey/issues/182
             return
+        # bring console tab to the front
+        self.get_widget('consoleTabs').setCurrentWidget(
+            self.get_widget('commandLogTab'))
+
         self.get_widget('commandLog').appendHtml(
-            f"<hr>{render_cmd_call(cmdname, cmdargs)}<hr>"
+            f"<hr>{render_cmd_call(cmdname, cmdargs, 'Running')}"
         )
 
     def _setup_stopped_cmdexec(
@@ -192,6 +236,10 @@ class GooeyApp(QObject):
         if ce is None:
             self.get_widget('statusbar').showMessage(f'Finished `{cmdname}`',
                                                      timeout=1000)
+            if not cmdname.startswith('gooey_'):
+                self.get_widget('commandLog').appendHtml(
+                    f"{render_cmd_call(cmdname, cmdargs, '-> Done')}<hr>"
+                )
         else:
             from datalad.support.exceptions import IncompleteResultsError
             if ce.tb.exc_type is IncompleteResultsError:
@@ -200,7 +248,8 @@ class GooeyApp(QObject):
                 error_hint = ""
             else:
                 error_hint = " (see error log for details)"
-            failed_msg = f"{render_cmd_call(cmdname, cmdargs)} <b>failed!</b>"
+            failed_msg = \
+                f"{render_cmd_call(cmdname, cmdargs, '-> Failed')}<hr>"
             # if a command crashes, state it in the statusbar
             self.get_widget('statusbar').showMessage(
                 f'`{cmdname}` failed{error_hint}')
@@ -221,25 +270,33 @@ class GooeyApp(QObject):
         if not self._cmdexec.n_running:
             self.main_window.setCursor(QCursor(Qt.ArrowCursor))
 
-    def deinit(self):
-        dlui.ui.set_backend(self._prev_ui_backend)
-        # restore any possible term prompt setup
-        for var, val in self._restore_env.items():
-            if val is not None:
-                environ[var] = val
+        # act on any pending close request
+        if self.__app_close_requested:
+            self.__app_close_requested = False
+            self.main_window.close()
 
     #@cached_property not available for PY3.7
     @property
     def main_window(self):
-        if not self._main_window:
-            self._main_window = load_ui('main_window')
-        return self._main_window
+        if self.__main_window is None:
+            self.__main_window = load_ui(
+                'main_window',
+                custom_widgets=[
+                    HistoryWidget,
+                    MetadataWidget,
+                    PropertyWidget,
+                ]
+            )
+            # hook into all events that the main window receives
+            # e.g. to catch close events and store window configuration
+            self.__main_window.installEventFilter(self)
+        return self.__main_window
 
-    def get_widget(self, name):
-        wgt_cls = GooeyApp._main_window_widgets.get(name)
+    def get_widget(self, name: str) -> QWidget:
+        wgt_cls = self._widgets.get(name)
         if not wgt_cls:
             raise ValueError(f"Unknown widget {name}")
-        wgt = self.main_window.findChild(wgt_cls, name=name)
+        wgt = cast(QWidget, self.main_window.findChild(wgt_cls, name=name))
         if not wgt:
             # if this happens, our internal _widgets is out of sync
             # with the UI declaration
@@ -247,7 +304,7 @@ class GooeyApp(QObject):
                 f"Could not locate widget {name} ({wgt_cls.__name__})")
         return wgt
 
-    def _set_root_path(self, path: Path):
+    def _set_root_path(self, path: Path = None):
         """Store the application root path and change PWD to it
 
         Right now this method can only be called once and only before the GUI
@@ -274,6 +331,11 @@ class GooeyApp(QObject):
                 # HOME is a better choice than CWD in most environments
                 path = Path.home()
 
+        path = Path(path)
+        if not path.is_absolute():
+            # internal machinery expects absolute paths.
+            # relative is relative to CWD
+            path = Path.cwd() / path
         chpwd(path)
         self._path = path
         # (re)init the browser
@@ -283,6 +345,51 @@ class GooeyApp(QObject):
     def rootpath(self):
         return self._path
 
+    def _edit_metadata(self):
+        """Private slot to pull up a metadata editor"""
+        action = self.sender()
+        if action is not None:
+            path, editor_type = action.data()
+        if path is None or editor_type is None:
+            raise ValueError(
+                'MetadataWidget.setup_for() called without a path or metadata '
+                'editor type')
+
+        tab = self.get_widget('metadataTab')
+        metadata_widget = self.get_widget('metadataTabWidget')
+        metadata_widget.setup_for(path=path, editor_type=editor_type)
+        self.show_help(metadata_widget.get_doc_text())
+        # open the metadata tab
+        self.get_widget('contextTabs').setCurrentWidget(tab)
+
+    def show_help(self, text: str):
+        hbrowser = self.get_widget('helpBrowser')
+        hbrowser.setPlainText(text)
+        # bring help tab to the front
+        self.get_widget('consoleTabs').setCurrentWidget(
+            self.get_widget('helpTab'))
+
+    def _start_file_manager(self):
+        """Private slot to start a platform-appropriate file manager"""
+        act = self.sender()
+        if not act:
+            return
+        path = act.data()
+        platform_name = platform.system()
+        if platform_name == 'Linux':
+            os.system(f'xdg-open "{path}"')
+        elif platform_name == 'Darwin':
+            os.system(f'open "{path}"')
+        elif platform_name == 'Windows':
+            os.startfile(str(path))
+
+    def _open_terminal(self):
+        """Private slot to open a terminal at the given location"""
+        act = self.sender()
+        if not act:
+            return
+        open_terminal_at_path(act.data())
+
     def _populate_datalad_menu(self):
         """Private slot to populate connected QMenus with dataset actions"""
         sender = self.sender()
@@ -299,69 +406,6 @@ class GooeyApp(QObject):
         self.get_widget('menuDatalad').aboutToShow.disconnect(
             self._populate_datalad_menu)
 
-    def _check_new_version(self):
-        self.get_widget('statusbar').showMessage(
-            'Checking latest version', timeout=2000)
-        try:
-            is_outdated, latest = check_outdated('datalad', dlversion)
-        except ValueError:
-            # thrown when one is in a development version (ie., more
-            # recent than the most recent release)
-            is_outdated = False
-            pass
-        mbox = QMessageBox.information
-        title = 'Version check'
-        msg = 'Your DataLad version is up to date.'
-        if is_outdated:
-            mbox = QMessageBox.warning
-            msg = f'A newer DataLad version {latest} ' \
-                  f'is available (installed: {dlversion}).'
-        mbox(self.main_window, title, msg)
-
-    def _get_issue_template(self):
-        mbox = QMessageBox.warning
-        title = 'Oooops'
-        msg = 'Please report unexpected or faulty behavior to us. File a ' \
-              'report with <a href="https://github.com/datalad/datalad-gooey/issues/new?template=issue_template.yml">' \
-              'datalad-gooey</a> or with <a href="https://github.com/datalad/datalad-gooey/issues/new?assignees=&labels=gooey&template=issue_template_gooey.yml">' \
-              'DataLad</a>'
-        mbox(self.main_window, title, msg)
-
-    def _get_help(self):
-        mbox = QMessageBox.information
-        title = 'I need help!'
-        msg = 'Find resources to learn more or ask questions here: <ul><li>' \
-              'About this tool: <a href=http://docs.datalad.org/projects/gooey/en/latest>DataLad Gooey Docs</a> </li>' \
-              '<li>General DataLad user tutorials: <a href=http://handbook.datalad.org> handbook.datalad.org</a> </li>' \
-              '<li>Live chat and weekly office hour: <a href="https://matrix.to/#/!NaMjKIhMXhSicFdxAj:matrix.org?via=matrix.waite.eu&via=matrix.org&via=inm7.de">' \
-              'Join us on Matrix</a></li></ul>'
-        mbox(self.main_window, title, msg)
-
-    def _get_info(self):
-        mbox = QMessageBox.information
-        title = 'About'
-        msg = 'DataLad and DataLad Gooey are free and open source software. ' \
-              'Read the <a href=https://doi.org/10.21105/joss.03262> paper' \
-              '</a>, or find out more at <a href=http://datalad.org>' \
-              'datalad.org</a>.'
-        mbox(self.main_window, title, msg)
-
-    def _get_diagnostic_info(self):
-        self.execute_dataladcmd.emit(
-            'wtf',
-            MappingProxyType(dict(
-                result_renderer='disabled',
-                on_failure='ignore',
-                return_type='generator',
-            )),
-            MappingProxyType(dict(
-                preferred_result_interval=0.2,
-                result_override=dict(
-                    secret_handshake=True,
-                ),
-            )),
-        )
-
     @Slot(Interface, list)
     def _app_cmdexec_results_handler(self, cls, res):
         if cls != WTF:
@@ -447,6 +491,60 @@ class GooeyApp(QObject):
                 action.setToolTip(description)
             suite_menu.addAction(action)
 
+    def _restore_configuration(self) -> None:
+        mw = self.main_window
+        # Restore prior configuration
+        self._qt_settings = QSettings("datalad", self.__class__.__name__)
+        mw.restoreGeometry(self._qt_settings.value('geometry'))
+        mw.restoreState(self._qt_settings.value('state'))
+
+        fs_browser: QWidget = self.get_widget('fsBrowser')
+        fs_browser.restoreGeometry(
+            self._qt_settings.value('geometry/fsBrowser'))
+        fs_browser.header().restoreState(
+            self._qt_settings.value('state/fsBrowser/header'))
+
+    def _store_configuration(self) -> None:
+        mw = self.main_window
+        # Store configuration of main elements we care storing
+        self._qt_settings.setValue('geometry', mw.saveGeometry())
+        self._qt_settings.setValue('state', mw.saveState())
+
+        fs_browser: QWidget = self.get_widget('fsBrowser')
+        self._qt_settings.setValue(
+            'geometry/fsBrowser', fs_browser.saveGeometry())
+        self._qt_settings.setValue(
+            'state/fsBrowser/header', fs_browser.header().saveState())
+
+    def eventFilter(self, watched, event):
+        if event.type() == QEvent.Close and watched is self.main_window:
+            if self._cmdexec.n_running:
+                # we ignore close events while exec threads are still running
+                # instead we set a flag to trigger another close
+                # event when a command exits
+                self.__app_close_requested = True
+                # prevent further commands from starting, even if already
+                # queued
+                self._cmdexec.shutdown()
+                self.get_widget('statusbar').showMessage(
+                    'Shutting down, waiting for pending commands to finish...')
+                event.ignore()
+                return True
+            self._store_configuration()
+            # undo UI backend
+            dlui.ui.set_backend(self._prev_ui_backend)
+            # restore any possible term prompt setup
+            for var, val in self._restore_env.items():
+                if val is not None:
+                    environ[var] = val
+            return super().eventFilter(watched, event)
+        elif event.type() in (QEvent.Destroy, QEvent.ChildRemoved):
+            # must catch this one or the access of `watched` in the `else`
+            # will crash the app, because it is already gone
+            return False
+        else:
+            return super().eventFilter(watched, event)
+
 
 def main():
     qtapp = QApplication(sys.argv)
